local addon = LibStub("AceAddon-3.0"):GetAddon("GCD")
local module = addon:NewModule("cast")
local media = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GCD")

local sin, cos, rad = math.sin, math.cos, math.rad
local GetTime = GetTime
local castStartTime, castEndTime, castDuration, castInterrupted, castLatency, castSent, isCasting
local castFrame
local options
local ringMod

local defaults = {
	profile = {
		castColor = {r=0, g=1, b=0, a=1},
		latencyColor = {r=1, g=0, b=0, a=1},
		interruptColor = {r=1, g=1, b=0, a=1},
		texture = "Spells\\AURA_01",
		spellText = {
			enabled = false,
			font = "Calibri",
			fontSize = 12
		},
		hideCastBar = false
	}
}

function module:OnInitialize()
	self.db = addon.db:RegisterNamespace("Cast", defaults)
	ringMod = addon:GetModule("ring", true)
end

function module:GetOptions()
	options = {
		name = L["Casttime"],
		type = "group",
		args = {
			hideCastBar = {
				name = L["Hide default castbar"],
				type = "toggle",
				disabled = function() return not addon.db.char.modules.cast end,
				get = function(info) return self.db.profile.hideCastBar end,
				set = function(info,val)
						self.db.profile.hideCastBar = val
						if val then
							CastingBarFrame:UnregisterAllEvents()
							CastingBarFrame:Hide()
						else
							CastingBarFrame:UnregisterAllEvents()
							CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START")
							CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
							CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
							CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
							CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED")
							CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
							CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
							CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
						end
					end,
				order = 11
			},
			spark = {
				name = L["Spark"],
				type = "header",
				order = 20
			},
			texture = {
				name = L["Texture"],
				type = "input",
				disabled = function() return not addon.db.char.modules.cast end,
				get = function(info) return self.db.profile.texture end,
				set = function(info, val)
							self.db.profile.texture = val
							self:ApplyOptions()
						end,
				order = 21
			},
			castColor = {
				name = L["Cast color"],
				type = "color",
				disabled = function() return not addon.db.char.modules.cast end,
				get = function(info) return self.db.profile.castColor.r, self.db.profile.castColor.g, self.db.profile.castColor.b, self.db.profile.castColor.a end,
				set = function(info, r, g, b, a)
							self.db.profile.castColor = {r=r, g=g, b=b, a=a}
							self:ApplyOptions()
						end,
				hasAlpha = true,
				order = 22
			},
			latencyColor = {
				name = L["Latency color"],
				type = "color",
				disabled = function() return not addon.db.char.modules.cast end,
				get = function(info) return self.db.profile.latencyColor.r, self.db.profile.latencyColor.g, self.db.profile.latencyColor.b, self.db.profile.latencyColor.a end,
				set = function(info, r, g, b, a)
							self.db.profile.latencyColor = {r=r, g=g, b=b, a=a}
							self:ApplyOptions()
						end,
				hasAlpha = true,
				order = 23
			},
			interruptColor = {
				name = L["Interrupt color"],
				type = "color",
				disabled = function() return not addon.db.char.modules.cast end,
				get = function(info) return self.db.profile.interruptColor.r, self.db.profile.interruptColor.g, self.db.profile.interruptColor.b, self.db.profile.interruptColor.a end,
				set = function(info, r, g, b, a)
							self.db.profile.interruptColor = {r=r, g=g, b=b, a=a}
							self:ApplyOptions()
						end,
				hasAlpha = true,
				order = 24
			},
			spellText = {
				name = L["Spell Text"],
				type = "header",
				order = 30
			},
			spellTextEnabled = {
				name = L["Enabled"],
				type = "toggle",
				disabled = function() return not addon.db.char.modules.cast end,
				get = function(info) return self.db.profile.spellText.enabled end,
				set = function(info,val)
						self.db.profile.spellText.enabled = val
						self:ApplyOptions()
					end,
				order = 31
			},
			font = {
				name = L["Font"],
				type = "select",
				disabled = function() return (not addon.db.char.modules.cast or not self.db.profile.spellText.enabled) end,
				dialogControl = 'LSM30_Font',
				get = function() return self.db.profile.spellText.font end,
				set = function(_, value)
							self.db.profile.spellText.font = value
							self:ApplyOptions()
						end,
				values = media:HashTable("font"),
				order = 32
			},
			fontSize = {
				name = L["Font Size"],
				type = "range",
				disabled = function() return (not addon.db.char.modules.cast or not self.db.profile.spellText.enabled) end,
				min = 1,
				max = 30,
				step = 1,
				get = function() return self.db.profile.spellText.fontSize end,
				set = function(_, value)
							self.db.profile.spellText.fontSize = value
							self:ApplyOptions()
						end,
				order = 33
			},
			misc = {
				name = L["Miscellaneous"],
				type = "header",
				order = 40
			},
			defaults = {
				name = L["Restore defaults"],
				type = "execute",
				disabled = function() return not addon.db.char.modules.cast end,
				func = function()
							self.db:ResetProfile()
							self:ApplyOptions()
						end,
				order = 41
			}
		}
	}
	return options
end

function module:OnEnable()
	self:RegisterEvent("UNIT_SPELLCAST_SENT")
	self:RegisterEvent("UNIT_SPELLCAST_START")
	self:RegisterEvent("UNIT_SPELLCAST_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
	self:RegisterEvent("UNIT_SPELLCAST_FAILED")
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	self:ApplyOptions()
end

function module:OnDisable()
	self:UnregisterAllEvents()
	self:Hide()
end

function module:Show()
	addon:Show("cast")
	if ringMod and ringMod:IsEnabled() then ringMod:Show("cast") end
	castFrame:Show()
end

function module:Hide()
	castFrame:Hide()
	if ringMod and ringMod:IsEnabled() then ringMod:Hide("cast") end
	addon:Hide("cast")
end

local function OnUpdate(self, elapsed)
	local castPerc = (1000 * GetTime() - castStartTime) / castDuration
	castPerc = castPerc < 1 and castPerc or 1
	if castPerc > 1-castLatency then
		self.spark:SetVertexColor(module.db.profile.latencyColor.r, module.db.profile.latencyColor.g, module.db.profile.latencyColor.b, module.db.profile.latencyColor.a) -- 1,0,0,1
	elseif castInterrupted then
		self.spark:SetVertexColor(module.db.profile.interruptColor.r, module.db.profile.interruptColor.g, module.db.profile.interruptColor.b, module.db.profile.interruptColor.a) -- 1,1,0,1
	else
		self.spark:SetVertexColor(module.db.profile.castColor.r, module.db.profile.castColor.g, module.db.profile.castColor.b, module.db.profile.castColor.a) -- 0,1,0,1
	end
	local angle = rad(90-castPerc*360)
	local x, y = cos(angle), sin(angle)
	self.spark:ClearAllPoints()
	self.spark:SetPoint('CENTER', self, 'CENTER',x*30,y*30)
	self.spark:Show()
	if castPerc >= 1 then
		module:Hide()
	end
end

function module:UNIT_SPELLCAST_SENT(_, unit)
	if unit ~= 'player' then return end
	castSent = GetTime()*1000
end

function module:UNIT_SPELLCAST_START(_, unit, spell)
	if unit ~= 'player' then return end
	castInterrupted = false
	_, _, _, icon, castStartTime, castEndTime = UnitCastingInfo(unit)
	local sendLag= (castSent and castSent > 0) and GetTime()*1000-castSent or 0
	castDuration=castEndTime-castStartTime
	sendLag = sendLag > castDuration and castDuration or sendLag
	castLatency = sendLag/castDuration
	if castFrame.spellText then
		castFrame.spellText:SetText(spell)
	end
	self:Show()
end

function module:UNIT_SPELLCAST_STOP(_, unit)
	if unit ~= 'player' then return end
	self:Hide()
end

function module:UNIT_SPELLCAST_FAILED(event, unit)
	if unit ~= 'player' then return end
	self:Hide()
end

function module:UNIT_SPELLCAST_INTERRUPTED(event, unit)
	if unit ~= 'player' then return end
	self:Hide()
end

function module:UNIT_SPELLCAST_DELAYED(event, unit)
	if unit ~= 'player' then return end
	castInterrupted = true
	_, _, _, _, castStartTime, castEndTime = UnitCastingInfo(unit)
end

function module:UNIT_SPELLCAST_CHANNEL_START(event,unit)
	if unit ~= 'player' then return end
	_, _, _, _, castStartTime, castEndTime = UnitChannelInfo(unit)
	local sendLag = (castSent and castSent > 0) and GetTime() * 1000 - castSent or 0
	castDuration = castEndTime - castStartTime
	sendLag = sendLag > castDuration and castDuration or sendLag
	castLatency = sendLag / castDuration
	if castFrame.spellText then
		castFrame.spellText:SetText(spell)
	end
	self:Show()
end

function module:UNIT_SPELLCAST_CHANNEL_STOP(event,unit)
	if unit ~= 'player' then return end
	self:Hide()
end

function module:UNIT_SPELLCAST_CHANNEL_UPDATE(event,unit)
	if unit ~= 'player' then return end
	castInterrupted = true
	_, _, _, _, castStartTime, castEndTime = UnitChannelInfo(unit)
end

function module:ApplyOptions()
	local anchor = addon.anchor
	if self:IsEnabled() then
		castFrame = castFrame or CreateFrame("Frame")
		castFrame:SetParent(anchor)
		castFrame:SetAllPoints()
		self:Hide()
		castFrame:SetScript('OnUpdate', OnUpdate)
		
		local spark = castFrame.spark or castFrame:CreateTexture(nil, 'DIALOG')
		spark:SetTexture(self.db.profile.texture) -- Spells\\AURA_01
		spark:SetVertexColor(self.db.profile.castColor.r, self.db.profile.castColor.g, self.db.profile.castColor.b, self.db.profile.castColor.a) -- 0,1,0,1
		spark:SetBlendMode('ADD')
		spark:SetWidth(30)
		spark:SetHeight(30)
		castFrame.spark = spark
		
		if self.db.profile.spellText then
			local spellText = castFrame.spellText or castFrame:CreateFontString(nil, "OVERLAY")
			spellText:ClearAllPoints()
			spellText:SetPoint("BOTTOM", castFrame, "TOP", 0, 5)
			spellText:SetFont(media:Fetch("font", self.db.profile.spellText.font), self.db.profile.spellText.fontSize)
			spellText:Show()
			castFrame.spellText = spellText
		elseif castFrame.spellText then
			castFrame.spellText:ClearAllPoints()
			castFrame.spellText:Hide()
		end
		
		if self.db.profile.hideCastBar then
			CastingBarFrame:UnregisterAllEvents()
			CastingBarFrame:Hide()
		else
			CastingBarFrame:UnregisterAllEvents()
			CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START")
			CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
			CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
			CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
			CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED")
			CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
			CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
			CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
		end
	end
end