local addon = LibStub("AceAddon-3.0"):GetAddon("GCD")
local module = addon:NewModule("gcd")
local L = LibStub("AceLocale-3.0"):GetLocale("GCD")

local sin, cos, rad = math.sin, math.cos, math.rad
local GetTime = GetTime
local spellNum
local gcdFrame
local options
local ringMod

local spells = {
	["DRUID"] = 5185,
	["PALADIN"] = 635,
	["PRIEST"] = 2050,
	["SHAMAN"] = 331,
	["WARRIOR"] = 6673,
	["DEATHKNIGHT"] = 45902,
	["HUNTER"] = 75,
	["MAGE"] = 1459,
	["WARLOCK"] = 687,
	["ROGUE"] = 1752,
}

local defaults = {
	profile = {
		color = {r=1, g=1, b=1, a=1},
		texture = "Spells\\AURA_01"
	}
}

function module:OnEnable()
	self:SPELLS_CHANGED()
	self:RegisterEvent("SPELLS_CHANGED")
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
end

function module:OnDisable()
	self:UnregisterAllEvents()
	self:Hide()
end

function module:OnInitialize()
	self.db = addon.db:RegisterNamespace("GCD", defaults)
	ringMod = addon:GetModule("ring", true)
end

function module:GetOptions()
	options = {
		name = "GCD",
		type = "group",
		args = {
			display = {
				name = L["Display"],
				type = "header",
				order = 10
			},
			texture = {
				name = L["Texture"],
				type = "input",
				disabled = function() return not addon.db.char.modules.gcd end,
				get = function(info) return self.db.profile.texture end,
				set = function(info, val)
							self.db.profile.texture = val
							self:ApplyOptions()
						end,
				order = 11
			},
			color = {
				name = L["Color"],
				type = "color",
				disabled = function() return not addon.db.char.modules.gcd end,
				get = function(info) return self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b, self.db.profile.color.a end,
				set = function(info, r, g, b, a)
							self.db.profile.color = {r=r, g=g, b=b, a=a}
							self:ApplyOptions()
						end,
				hasAlpha = true,
				order = 12
			},
			misc = {
				name = L["Miscellaneous"],
				type = "header",
				order = 20
			},
			defaults = {
				name = L["Restore defaults"],
				type = "execute",
				disabled = function() return not addon.db.char.modules.gcd end,
				func = function()
							self.db:ResetProfile()
							self:ApplyOptions()
						end,
				order = 21
			}
		}
	}
	return options
end

function module:Show()
	addon:Show("gcd")
	if ringMod and ringMod:IsEnabled() then ringMod:Show("gcd") end
	gcdFrame:Show()
end

function module:Hide()
	gcdFrame:Hide()
	if ringMod and ringMod:IsEnabled() then ringMod:Hide("gcd") end
	addon:Hide("gcd")
end

local function OnUpdate(self, elapsed)
	local gcdPerc = (GetTime() - self.startTime) / self.duration
	gcdPerc = gcdPerc < 1 and gcdPerc or 1
	local angle = rad(90-gcdPerc*360)
	local x, y = cos(angle), sin(angle)
	self.texture:ClearAllPoints()
	self.texture:SetPoint('CENTER', self, 'CENTER',x*25,y*25)
	self.texture:Show()
	if gcdPerc >= 1 then
		module:Hide()
	end
end

function module:ApplyOptions()
	local anchor = addon.anchor
	if self:IsEnabled() then
		if not gcdFrame then
			gcdFrame = CreateFrame("Frame")
			gcdFrame:SetParent(anchor)
			gcdFrame:SetAllPoints()
			gcdFrame.texture = gcdFrame:CreateTexture(nil, 'DIALOG')
		end
		self:Hide()
		gcdFrame:SetScript('OnUpdate', OnUpdate)
		local texture = gcdFrame.texture
		texture:SetTexture(self.db.profile.texture) -- Spells\\AURA_01
		texture:SetBlendMode('ADD')
		texture:SetVertexColor(self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b, self.db.profile.color.a) -- 1,1,1
		texture:SetWidth(30)
		texture:SetHeight(30)
	end
end

function module:ACTIONBAR_UPDATE_COOLDOWN()
	if spellNum then
		local start, dur = GetSpellCooldown(spellNum, BOOKTYPE_SPELL)
		if dur > 0 and dur <= 1.5 then
			gcdFrame.startTime = start
			gcdFrame.duration = dur
			self:Show()
		end
	end
end

function module:SPELLS_CHANGED()
	local _, class = UnitClass("player")
	spellNum = addon:GetSpellPosInSpellbook(GetSpellInfo(spells[class]))
	self:ApplyOptions()
end